/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 * Copyright (C) 2005 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation.
 * See the COPYING file for more information.
 *
 **********************************************************************
 *
 * Don't include this file, rather include from <geos/operation/class.h>
 *
 **********************************************************************/

#ifndef GEOS_OPERATION_H
#define GEOS_OPERATION_H

namespace geos {

/// Provides classes for implementing operations on geometries
namespace operation { // geos.operation

} // namespace geos.operation
} // namespace geos

#include <geos/operation/GeometryGraphOperation.h>
#include <geos/operation/IsSimpleOp.h>

#endif
