/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2006 Refractions Research Inc.
 * Copyright (C) 2001-2002 Vivid Solutions Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation.
 * See the COPYING file for more information.
 *
 **********************************************************************/

#ifndef GEOS_INDEXQUADTREE_H
#define GEOS_INDEXQUADTREE_H

namespace geos {
namespace index { // geos.index

/// Contains classes that implement a Quadtree spatial index
namespace quadtree { // geos.index.quadtree

} // namespace geos.index.quadtree
} // namespace geos.index
} // namespace geos

//#include <geos/index/quadtree/DoubleBits.h>
//#include <geos/index/quadtree/IntervalSize.h>
//#include <geos/index/quadtree/Key.h>
//#include <geos/index/quadtree/Node.h>
#include <geos/index/quadtree/NodeBase.h>
#include <geos/index/quadtree/Quadtree.h>
#include <geos/index/quadtree/Root.h>

#endif

